"""DO NOT EDIT.

This file was autogenerated. Do not edit it by hand,
since your modifications would be overwritten.
"""

from keras_hub.src.utils.imagenet.imagenet_utils import (
    decode_imagenet_predictions,
)
